from ekispert.models.section_separator import SectionSeparator
from ekispert.models.teiki_route_section import TeikiRouteSection
from ..base import Base

class TeikiRoute(Base):
  def __init__(self, data = None):
    super().__init__()
    if data is None:
      return
    self.sets(data)

  def sets(self, data: dict):
    for key in data:
      self.set(key, data[key])

  def set(self, key: str, value: any):
    match key.lower():
      case "sectionseparator":
        self.section_separator = SectionSeparator(value)
      case "teikiroutesection":
        teiki_route_sections = self.get_as_array(value)
        if teiki_route_sections is not None and len(teiki_route_sections) > 0:
          self.teiki_route_sections = []
          for teiki_route_section in teiki_route_sections:
            self.teiki_route_sections.append(TeikiRouteSection(teiki_route_section))
      case _:
        raise ValueError(f"key: {key} is not defined in TeikiRoute")



from ..base import Base

class PassStatus(Base):
  def __init__(self, data = None):
    super().__init__()
    if data is None:
      return
    self.sets(data)

  def sets(self, data: dict):
    for key in data:
      self.set(key, data[key])

  def set(self, key: str, value: any):
    match key.lower():
      case "kind":
        self.kind = value
      case "selected":
        if value == "true":
          self.selected = True
        else:
          self.selected = False
      case "fromlineindex":
        self.from_line_index = int(value)
      case "tolineindex":
        self.to_line_index = int(value)
      case "teiki1index":
        self.teiki1_index = int(value)
      case "teiki3index":
        self.teiki3_index = int(value)
      case "teiki6index":
        self.teiki6_index = int(value)
      case "teiki12index":
        self.teiki12_index = int(value)
      case "comment":
        self.comment = value
      case "name":
        self.name = value
      case "type":
        self.type = value
      case _:
        raise ValueError(f"key: {key} is not defined in PassStatus")


from ..base import Base

from collections import namedtuple

PriceRelation = namedtuple('PriceRelation', 'kind')

class Relation(Base):
  def __init__(self, data = None):
    super().__init__()
    if data is None:
      return
    self.sets(data)

  def sets(self, data: dict):
    for key in data:
      self.set(key, data[key])

  def set(self, key: str, value: any):
    match key.lower():
      case "name":
        self.name = value
      case "pricerelation":
        # value is array or not
        value = self.get_as_array(value)
        self.price_relations = []
        for v in value:
          if v is not None:
            self.price_relations.append(PriceRelation(
              kind = v["kind"]
            ))
      case _:
        raise ValueError(f"key: {key} is not defined in Relation")


from ..base import Base

class RepaymentTicket(Base):
  def __init__(self, data = None):
    super().__init__()
    if data is None:
      return
    self.sets(data)

  def sets(self, data: dict):
    for key in data:
      self.set(key, data[key])

  def set(self, key: str, value: any):
    match key.lower():
      case "feepricevalue":
        if value is not None:
          self.fee_price_value = int(value)
      case "repaypricevalue":
        if value is not None:
          self.repay_price_value = int(value)
      case "state":
        if value is not None:
          self.state = int(value)
      case "usedpricevalue":
        if value is not None:
          self.used_price_value = int(value)
      case "calculatetarget":
        if value is not None:
          self.calculate_target = bool(value)
      case "toteikiroutesectionindex":
        if value is not None:
          self.to_teiki_route_section_index = int(value)
      case "fromteikiroutesectionindex":
        if value is not None:
          self.from_teiki_route_section_index = int(value)
      case "validityperiod":
        if value is not None:
          self.validity_period = int(value)
      case "paypricevalue":
        if value is not None:
          self.pay_price_value = int(value)
      case "changeablesection":
        if value is not None:
          self.changeable_section = bool(value)
      case _:
        raise ValueError(f"key: {key} is not defined in RepaymentTicket")



from ekispert.models.line import Line
from ekispert.models.point import Point
from ..base import Base

class Route(Base):
  def __init__(self, data = None):
    super().__init__()
    if data is None:
      return
    self.sets(data)

  def sets(self, data: dict):
    for key in data:
      self.set(key, data[key])

  def set(self, key: str, value: any):
    match key.lower():
      case "distance":
        self.distance = int(value)
      case "exhaustco2":
        self.exhaust_co2 = int(value)
      case "exhaustco2atpassengercar":
        self.exhaust_co2_at_passenger_car = int(value)
      case "timeOnboard":
        self.time_on_board = int(value)
      case "timeother":
        self.time_other = int(value)
      case "timewalk":
        self.time_walk = int(value)
      case "transfercount":
        self.transfer_count = int(value)
      case "line":
        value = self.get_as_array(value)
        self.lines = []
        for v in value:
          if v is not None:
            self.lines.append(Line(v))
      case "point":
        value = self.get_as_array(value)
        self.points = []
        for v in value:
          if v is not None:
            self.points.append(Point(v))
      case "timeonboard":
        self.time_on_board = int(value)
      case _:
        raise ValueError(f"key: {key} is not defined in Route")


from datetime import datetime

from ekispert.models.repayment_list import RepaymentList
from ekispert.models.teiki_route import TeikiRoute
from ..base import Base
from typing import List, Literal

class CourseRepaymentResponse:
  repayment_list: RepaymentList
  teiki_route: TeikiRoute
  def __init__(self, repayment_list: RepaymentList, teiki_route: TeikiRoute):
    self.repayment_list = repayment_list
    self.teiki_route = teiki_route

ValidityPeriod = Literal[1, 3, 6, 12]
class CourseRepaymentQuery(Base):
  base_path: str = '/v1/json/course/repayment'

  def __init__(self, client):
    super().__init__()
    self.client = client

    self.serialize_data : str = ''
    self.check_engine_version : bool = True
    self.start_date : datetime = datetime.now()
    self.buy_date : datetime = datetime.now()
    self.repayment_date : datetime = datetime.now()
    self.validity_period : ValidityPeriod = 6
    self.change_section : bool = False
    self.separator : List[str] = []

  def execute(self) -> CourseRepaymentResponse:
    data = self.client.get(self.base_path, self.generate_params())
    result_set = data['ResultSet']
    repayment_list = result_set['RepaymentList']
    teiki_route = result_set['TeikiRoute']
    if repayment_list is None and teiki_route is None:
      return CourseRepaymentResponse(
        repayment_list=None,
        teiki_route=None,
      )
    return CourseRepaymentResponse(
      repayment_list=RepaymentList(repayment_list),
      teiki_route=TeikiRoute(teiki_route),
    )

  def generate_params(self) -> dict:
    params = {
      'key': self.client.api_key,
    }
    if self.serialize_data == '':
      raise ValueError('serialize_data is required')
    params['serializeData'] = self.serialize_data
    params['checkEngineVersion'] = self.get_as_boolean_string(self.check_engine_version)
    if self.start_date:
      params['startDate'] = self.start_date.strftime('%Y%m%d')
    if self.buy_date:
      params['buyDate'] = self.buy_date.strftime('%Y%m%d')
    if self.repayment_date:
      params['repaymentDate'] = self.repayment_date.strftime('%Y%m%d')
    if self.validity_period:
      params['validityPeriod'] = self.validity_period
    params['changeSection'] = self.get_as_boolean_string(self.change_section)
    if self.separator is not None:
      params['separator'] = ','.join(self.separator)
    return params



from setuptools import setup, find_packages

setup(
  name='ekispert',
  version='0.2.0',
  packages=find_packages(),
  install_requires=[
    # 依存パッケージをここに列挙
    'requests',
  ],
  include_package_data=True,
  entry_points={
    'console_scripts': [
      'my_command=my_package.module:main_function',
    ],
  },
  author='Atsushi Nakatsugawa',
  author_email='atsushi@moongift.co.jp',
  description='SDK for Ekispert API',
  long_description=open('README.md').read(),
  long_description_content_type='text/markdown',
  url='https://github.com/EkispertMania/python_sdk',
  classifiers=[
      'Programming Language :: Python :: 3',
      'License :: OSI Approved :: MIT License',
      'Operating System :: OS Independent',
  ],
  python_requires='>=3.6',
)



from ..base import Base
from ..models.point import Point
from typing import List, Literal

TrafficType = Literal["train", "plane", "ship", "bus", "walk", "strange"]
NameMatchType = Literal["forward", "partial"]

class StationLightQuery(Base):
  base_path: str = '/v1/json/station/light'

  def __init__(self, client):
    super().__init__()
    self.client = client
    self.name : str = None
    self.name_match_type: NameMatchType = 'partial'
    self.code : int = None
    self.types : List[TrafficType] = None
    self.prefecture_codes : List[int] = None
    self.corporation_binds : List[str] = None
    self.community_bus : str = 'contain'

  def execute(self) -> List[Point]:
    data = self.client.get(self.base_path, self.generate_params())
    result_set = data['ResultSet']
    points = self.get_as_array(result_set['Point'])
    if len(points) == 0:
      return []
    results = []
    for point in points:
      results.append(Point(point))
    return results

  def generate_params(self) -> dict:
    params = {
      'key': self.client.api_key,
    }
    if self.name:
      params['name'] = self.name
    if self.name_match_type:
      params['nameMatchType'] = self.name_match_type
    if self.code:
      params['code'] = self.code
    if self.types:
      params['type'] = ':'.join(self.types)
    if self.prefecture_codes:
      params['prefectureCode'] = ':'.join(map(str, self.prefecture_codes))
    if self.corporation_binds:
      params['corporationBind'] = ':'.join(self.corporation_binds)
    if self.community_bus:
      params['communityBus'] = self.community_bus
    return params


from urllib.parse import urljoin, urlencode

from ekispert.queries.course_repayment import CourseRepaymentQuery
from ekispert.queries.course_plain import CoursePlainQuery
from .queries.station import StationQuery
from .queries.station_light import StationLightQuery
import requests

class Ekispert:
  base_url = 'https://api.ekispert.jp'

  def __init__(self, api_key):
    self.api_key = api_key
	
  def get(self, path, params):
    # requst to Ekispert API
    full_url = urljoin(self.base_url, path)
    # クエリパラメータをエンコード
    query_string = urlencode(params)
    # クエリパラメータを含む完全なURLを作成
    full_url_with_params = f"{full_url}?{query_string}"
    headers = {'Accept': 'application/json'}
    response = requests.get(full_url_with_params, headers=headers)
    if response.status_code == 200:
      try:
        data = response.json()  # JSONレスポンスを辞書型に変換
        return data
      except ValueError:
        print("Response content is not valid JSON")
    else:
      print(f"Request failed with status code {response.text}")

  def stationQuery(self) -> StationQuery:
    return StationQuery(self)

  def stationLightQuery(self) -> StationLightQuery:
    return StationLightQuery(self)

  def coursePlainQuery(self) -> CoursePlainQuery:
    return CoursePlainQuery(self)

  def courseRepaymentQuery(self) -> CourseRepaymentQuery:
    return CourseRepaymentQuery(self)


from ekispert.models.geo_point import GeoPoint
from ..base import Base
from collections import namedtuple

DateTime = namedtuple('DateTime', 'operation')
Gate = namedtuple('Gate', 'geo_point name')
class OperationalState(Base):
  def __init__(self, data = None):
    super().__init__()
    if data is None:
      return
    self.sets(data)

  def sets(self, data: dict):
    for key in data:
      self.set(key, data[key])

  def set(self, key: str, value: any):
    match key.lower():
      case "no":
        self.no = value
      case "datetime":
        if value["operation"] is not None:
          self.datetime = DateTime(
            operation = value["operation"])
      case "gate":
        self.gate = Gate(
          geo_point = GeoPoint(value["GeoPoint"]),
          name = value["Name"])
      case "type":
        self.type = value
      case _:
        raise ValueError(f"key: {key} is not defined in ArrivalState")
